# Copyright by Risto A. Paju (teknohog at iki.fi) 2014-2015

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#######################################################################

# Workspace coordinates: x, n

# Work unit sizes
#quantum = [2000, 2000]

# 2e5x2e5 box finished -- multiply WU size by 20x20 for 3..5 h units
# with the recent code speedup
quantum = [40000, 40000]
wulimits = [100, 100]

# Absolute minima -- quantum counting still starts at 0
infimum = [2, 29]

http_port = 1235

random_overstep = True

# Generate initial work space. This only sets some sensible maxima,
# before filtering out the finished units.
#wulimits = [100, 100]

def wu_generate(wu):
    global quantum, infimum

    # PredaYann1.ps, Lemma 3: x >= 10^6. This is more easily fixed
    # here, and since it aligns with our quantum, this limit can be
    # easily removed later if need be.
    #wu = (wu[0] + 500, wu[1])

    wu = (wu[0] + 25, wu[1])
    
    cmin = map(lambda x, y: x*y, wu, quantum)
    cmax = map(lambda x, y: (x+1)*y, wu, quantum)

    cmin = map(max, cmin, infimum)

    # Reorder for clients
    l = map(list, zip(cmin, cmax))
    # Regular sum() won't work for list concatenation
    output = reduce(lambda x, y: x+y, l)

    # Checksum placeholder
    output.append(0)
    
    return text_encode(output)

def wu_parse(wutext, unfinished = False):
    global quantum, infimum

    # This is really about checking the validity of the work, not so
    # much parsing the numbers for any further use. Try/except is used
    # to handle malformed input.
   
    try:
        # wranges = [xmin, xmax, nmin, nmax]
        wranges = text_decode(wutext)

        # Ignore the checksum at the end
        lower = wranges[::2][:-1]
        
        upper = wranges[1::2]

        if unfinished:
            # Move xmin down to the original start of the WU. The
            # int() is not necessary in Python, but it makes the
            # down-rounding logic clearer.
            lower[0] = max(infimum[0], int(lower[0]/quantum[0]) * quantum[0])
            
        # A single error will fail the entire input. The logic is
        # easier if we look for positive results, and fail for
        # everything else.

        for i in [0, 1]:
            if upper[i] % quantum[i] == 0:
                pass
            else:
                return ()

            if lower[i] >= quantum[i] and upper[i] == lower[i] + quantum[i]:
                pass
            elif lower[i] == infimum[i] and upper[i] == quantum[i]:
                pass
            else:
                return ()

        # Now that we have a full range, it is enough to use the upper ones
        return tuple(map(lambda x, y: x/y - 1, upper, quantum))
        
    except:
        return ()

# The dashed format is safer/easier for URLs etc. The crunchers can
# still use spaced format for easier command line... well, easy to
# change them too, to keep the whole simpler.

# Use try/except to handle malformed content

def text_decode(s):
    try:
        # Return a list of numbers
        return map(int, s.split("-"))
    except:
        return []

def text_encode(l):
    # Write a list of numbers
    return "-".join(map(str, l))
    
def write_file(filename, l, mode = "a"):
    # Append mode is best for results, so keep that as default
    content = str(l) + "\n"
    File = open(filename, mode)
    File.write(content)
    File.close()

def file_readlines(f):
    File = open(f, "r")
    contents = File.readlines()
    File.close()

    # Remove newlines etc.
    contents = map(lambda s: s.strip(), contents)
    
    return contents

def solution_verify(sol):
    # Filter against obvious math errors in the worker
        
    r = text_decode(sol)
    if len(r) == 4:
        x, y, n, q = r

        if (x**n - 1)/(x - 1) == y**q:
            return "verified"
        else:
            return "failed verification"

    else:
        return "invalid"
