#!/usr/bin/env julia

# Copyright by Risto A. Paju (teknohog at iki.fi) 2014

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#######################################################################

# Test the root_find method for false negatives and positives. For
# example, if there is not enough floating point precision,
# (y^q + 1)^(1/q) might match with y, thus producing a false negative.

function root_test(x, q)
    s = BigInt(x)

    by = BigInt(round(s^(1/q)))

    byq = by^q

    diff = s - byq

    #print(diff, "\n")
    
    #if byq == s
    if diff == 0
        return by
    else
        return 0
    end
end

nmin = 4250
nmax = 5450690

errors = 0

for i in 1:10
    for delta in [-1, 0, 1]
        y = BigInt(rand(nmin:nmax))
        q = rand(nmin:nmax)

        # Test for both false positives and false negatives with the delta
        x = y^q + delta

        print("Testing ", y, "^", q, " + ", delta, "\n")
        
        res = root_test(x, q)
        
        if (delta == 0 && res != y) || (delta != 0 && res == y)
            errors += 1
            print(y, " ", q, " ", delta, " Failed")
        end
    end
end

if errors == 0
    print("Test passed.\n")
else
    print("Test failed!\n")
end
